/*
	Nabaztag API
	Copyright 2007 Shannon Hicks
	http://www.iotashan.com/nabaztagexplorer/
	
	Released for use under GPL v2
	http://www.gnu.org/copyleft/gpl.html
*/
package org.iotashan.nabaztagAPI {
	import flash.events.EventDispatcher;
	
	import mx.controls.Alert;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;
	
	/**
	 * Dispatched when the API comes back, regardless of
	 * what the XML result says.
	*/
	[Event(name="apiResult", type=APIResultEvent)]
	/**
	 * Dispatched if the http result is a fault. Usually
	 * means that the Nabaztag servers are down
	*/
	[Event(name=FaultEvent.FAULT, type=mx.rpc.FaultEvent)]
	
	/**
	 * <p>The NabaztagAPIService is the core of this
	 * API Library. This is where you gather all
	 * the data for your API call, and actually make
	 * the API call.</p>
	 * 
	 * <p>For more info on the Nabaztag API, go to:
	 * <a href="http://api.nabaztag.com/" target="_blank">http://api.nabaztag.com/</a></p>
	*/
	public class NabaztagAPIService extends EventDispatcher {
		private var _sn:String;
		private var _token:String;
		private var _idmessage:String;
		private var _nabcast:String;
		private var _nabcasttitle:String;
		private var _posright:Number = NO_NUMERIC_VALUE;
		private var _posleft:Number = NO_NUMERIC_VALUE;
		private var _ears:Boolean;
		private var _idapp:String;
		private var _voice:NabaztagVoice;
		private var _tts:String;
		private var _speed:Number = NO_NUMERIC_VALUE;
		private var _pitch:Number = NO_NUMERIC_VALUE;
		private var _chor:NabaztagChoreography;
		private var _chortitle:String;
		private var _ttlive:Number = NO_NUMERIC_VALUE;
		private var _action:Number = NO_NUMERIC_VALUE;
		private var _urlList:String;
		
		/**
		 * String for the NabaztagAPIResultEvent type.
		*/
		public const API_RESULT_EVENT:String = "apiResult";
		/**
		 * A numeric value to represent that a numeric value has not been selected
		*/
		public const NO_NUMERIC_VALUE:Number = -1;
		/**
		 * The URL to the Nabaztag servers. Currently it's only used for
		 * display purposes, because the crossdomain.xml file is not open
		*/
		public const API_URL:String = "http://api.nabaztag.com/vl/FR/api.jsp?";
		
		// Action constants. These are hard-coded, and might be out of date.
		private const ACTION_GET_SOUND_PREVIEW_URL:Number = 1;
		private const ACTION_GET_FRIENDS:Number = 2;
		private const ACTION_GET_INBOX:Number = 3;
		private const ACTION_GET_TIMEZONE:Number = 4;
		private const ACTION_GET_SIGNATURE:Number = 5;
		private const ACTION_GET_BLACKLIST:Number = 6;
		private const ACTION_GET_SLEEP_STATUS:Number = 7;
		private const ACTION_GET_RABBIT_VERSION:Number = 8;
		private const ACTION_GET_TTS_LANGUAGES_VOICES:Number = 9;
		private const ACTION_GET_NAME:Number = 10;
		private const ACTION_GET_SELECTED_LANGUAGES:Number = 11;
		private const ACTION_GET_URLPLAY_PREVIEW_URL:Number = 12;
		private const ACTION_SET_RABBIT_SLEEP:Number = 13;
		private const ACTION_SET_RABBIT_AWAKE:Number = 14;
		
		/**
		 * Constructor method.
		*/
		public function NabaztagAPIService(sn:String,token:String) {
			super();
			// required fields
			_sn = sn;
			_token = token;
		}
		
		// getters and setters
		/**
		 * The serial number of your rabbit. If you're reading this,
		 * you'll probably notice that it's actually the MAC address.
		 * It's on the bottom of your rabbit.
		*/
		public function get sn():String {
			return _sn;
		}
		
		/**
		 * @private
		*/
		public function set sn(value:String):void {
			_sn = value;
		}
		
		/**
		 * Your rabbit's token. If you don't know it, you can find it
		 * at <a href="http://my.nabaztag.com" target="_blank">http://my.nabaztag.com</a>
		 * on your preferences page. It's in the Nabazphere pod. You will
		 * also need to check the checkbox that says
		 * "I want to receive the services and applications of the Nabazphere on my Nabaztag."
		*/
		public function get token():String {
			return _token;
		}
		
		/**
		 * @private
		*/
		public function set token(value:String):void {
			_token = value;
		}
		
		/**
		 * The number of the message to send. This number can refer to a
		 * message in the Library or a personal MP3 file that you have
		 * downloaded. You find this identification number under the title
		 * of the track you are listening to.
		*/
		public function get idmessage():String {
			return _idmessage;
		}
		
		/**
		 * @private
		*/
		public function set idmessage(value:String):void {
			_idmessage = value;
		}
		
		/**
		 * Id of your nabcast. (if you want to publish a content in your nabcast)
		*/
		public function get nabcast():String {
			return _nabcast;
		}
		
		/**
		 * @private
		*/
		public function set nabcast(value:String):void {
			_nabcast = value;
		}
		
		/**
		 * Title of the post in your nabcast.
		*/
		public function get nabcasttitle():String {
			return _nabcasttitle;
		}
		
		/**
		 * @private
		*/
		public function set nabcasttitle(value:String):void {
			_nabcasttitle = value;
		}
		
		/**
		 * Position of the right ear between 0 and 16. (0 = ear vertical)
		*/
		public function get posright():Number {
			return _posright;
		}
		
		/**
		 * @private
		*/
		public function set posright(value:Number):void {
			_posright = value;
		}
		
		/**
		 * Position of the left ear between 0 and 16. (0 = ear vertical)
		*/
		public function get posleft():Number {
			return _posleft;
		}
		
		/**
		 * @private
		*/
		public function set posleft(value:Number):void {
			_posleft = value;
		}
		
		/**
		 * Boolean that decides to send the position of the ears to your Nabaztag.
		*/
		public function get ears():Boolean {
			return _ears;
		}
		
		/**
		 * @private
		*/
		public function set ears(value:Boolean):void {
			_ears = value;
		}
		
		/**
		 * This is your application ID. It will allow you to authenticate
		 *  the event's transmitter. This parameter is not yet in service.
		*/
		public function get idapp():String {
			return _idapp;
		}
		
		/**
		 * @private
		*/
		public function set idapp(value:String):void {
			_idapp = value;
		}
		
		/**
		 * Allows you to choose the voice that will read the message.
		*/
		public function get voice():NabaztagVoice {
			return _voice;
		}
		
		/**
		 * @private
		*/
		public function set voice(value:NabaztagVoice):void {
			_voice = value;
		}
		
		/**
		 * (Text To Speech) String that will be converted into speech
		 * and read by your rabbit.
		*/
		public function get tts():String {
			return _tts;
		}
		
		/**
		 * @private
		*/
		public function set tts(value:String):void {
			_tts = value;
		}
		
		/**
		 * Allows you to choose the reading speed of the message.Values ranging from 1 to 32000. Default is 100.
		*/
		public function get speed():Number {
			return _speed;
		}
		
		/**
		 * @private
		*/
		public function set speed(value:Number):void {
			_speed = value;
		}
		
		/**
		 * Allows you to modulate the voice's frequency. Values ranging from 1 to 32000. Default is 100.
		*/
		public function get pitch():Number {
			return _pitch;
		}
		
		/**
		 * @private
		*/
		public function set pitch(value:Number):void {
			_pitch = value;
		}
		
		/**
		 * Sending a choreography to your Nabaztag.
		*/
		public function get chor():NabaztagChoreography {
			return _chor;
		}
		
		/**
		 * @private
		*/
		public function set chor(value:NabaztagChoreography):void {
			_chor = value;
		}
		
		/**
		 * The name of the choreography.
		*/
		public function get chortitle():String {
			return _chortitle;
		}
		
		/**
		 * @private
		*/
		public function set chortitle(value:String):void {
			_chortitle = value;
		}
		
		/**
		 * Allows you to define the length of time you want a message 
		 * to remain on the site (in seconds). By default, the message 
		 * will be stored for a period of two months.
		*/
		public function get ttlive():Number {
			return _ttlive;
		}
		
		/**
		 * @private
		*/
		public function set ttlive(value:Number):void {
			_ttlive = value;
		}
		
		/**
		 * The action you wish to use to get data about the API
		 * or your Rabbit. Currently accepts numeric values from 1 to 14.
		 * For more information, see the Nabaztag API Docs.
		*/
		public function get action():Number {
			return _action;
		}
		
		/**
		 * @private
		*/
		public function set action(value:Number):void {
			_action = value;
		}
		
		/**
		 * URL address of the MP3 file that you want your Nabaztag/tag to read. 
		 * If you wish to play several MP3 files in a row, you can can seperate 
		 * them with the pipe character « | ».
		*/
		public function get urlList():String {
			return _urlList;
		}
		
		/**
		 * @private
		*/
		public function set urlList(value:String):void {
			_urlList = value;
		}
		
		/**
		 * Returns the Nabaztag API URL. We don't include any properties with null values.
		*/
		public function getURL():String {
			// set base URL
			var thisURL:String = API_URL;
			
			// conditinally add in params
			if (_tts != null && _tts.length > 0) thisURL += "&tts=" + _tts;
			if (_idmessage != null && _idmessage.length > 0) thisURL += "&idmessage=" + _idmessage;
			if (_nabcast != null && _nabcast.length > 0) thisURL += "&nabcast=" + _nabcast;
			if (_urlList != null && _urlList.length > 0) thisURL += "&urlList=" + _urlList;
			if (_nabcasttitle != null && _nabcasttitle.length > 0) thisURL += "&nabcasttitle=" + _nabcasttitle;
			if (_ears) {
				thisURL += "&ears=" + "ok";
				if (_posleft != NO_NUMERIC_VALUE) thisURL += "&posleft=" + _posleft.toString();
				if (_posright != NO_NUMERIC_VALUE) thisURL += "&posright=" + _posright.toString();
			}
			if (_idapp != null && _idapp.length > 0) thisURL += "&idapp=" + _idapp;
			if (_voice != null) thisURL += "&voice=" + _voice.command;
			if (_speed != NO_NUMERIC_VALUE && _speed > 0) thisURL += "&speed=" + _speed.toString();
			if (_pitch != NO_NUMERIC_VALUE && _pitch > 0) thisURL += "&pitch=" + _pitch.toString();
			if (_chor != null) thisURL += "&chor=" + _chor.getChoreography();
			if (_chortitle != null && _chortitle.length > 0) thisURL += "&chortitle=" + _chortitle;
			if (_ttlive != NO_NUMERIC_VALUE) thisURL += "&ttlive=" + _ttlive.toString();
			if (_action != NO_NUMERIC_VALUE) thisURL += "&action=" + _action.toString();
			
			// removing the first "&" here is far easier than inline above
			thisURL = thisURL.replace("?&","?");
			
			return thisURL;
		}
		
		/**
		 * Sends the command(s) to the API. We don't send any properties with null values.
		*/
		public function send():void {
			// create the service
			var thisRequest:HTTPService = new HTTPService();
			var urlParams:Object = new Object();
			
			// set the request defaults
			thisRequest.url = API_URL;
			thisRequest.method = "GET";
			thisRequest.resultFormat = "xml";
			thisRequest.addEventListener(ResultEvent.RESULT,onResult);
			thisRequest.addEventListener(FaultEvent.FAULT,onFault);
			
			// set the sn & token params
			urlParams.sn = _sn;
			urlParams.token = _token;
			
			// set the standard params if they exist
			if (_tts != null && _tts.length > 0) urlParams.tts = _tts;
			if (_idmessage != null && _idmessage.length > 0) urlParams.idmessage = _idmessage;
			if (_nabcast != null && _nabcast.length > 0) urlParams.nabcast = _nabcast;
			if (_urlList != null && _urlList.length > 0) urlParams.urlList = _urlList;
			if (_nabcasttitle != null && _nabcasttitle.length > 0) urlParams.nabcasttitle = _nabcasttitle;
			if (_ears) {
				urlParams.ears = "ok";
				if (_posleft != NO_NUMERIC_VALUE) urlParams.posleft = _posleft;
				if (_posright != NO_NUMERIC_VALUE) urlParams.posright = _posright;
			}
			if (_idapp != null && _idapp.length > 0) urlParams.idapp = _idapp;
			if (_voice != null) urlParams.voice = _voice.command;
			if (_speed != NO_NUMERIC_VALUE && _speed > 0) urlParams.speed = _speed;
			if (_pitch != NO_NUMERIC_VALUE && _pitch > 0) urlParams.pitch = _pitch;
			if (_chor != null) urlParams.chor = _chor.getChoreography();
			if (_chortitle != null && _chortitle.length > 0) urlParams.chortitle = _chortitle;
			if (_ttlive != NO_NUMERIC_VALUE) urlParams.ttlive = _ttlive;
			if (_action != NO_NUMERIC_VALUE) urlParams.action = _action;
			
			// send the API url request
			thisRequest.send(urlParams);
		}
		
		private function onResult(event:ResultEvent):void {
			var myXML:XML = new XML(event.result);
			var thisResult:NabaztagAPIResultEvent = new NabaztagAPIResultEvent(myXML,"apiResult",true);
			dispatchEvent(thisResult);
		}
		
		private function onFault(event:FaultEvent):void {
			Alert.show("Error sending request. The Nabaztag API server is probably down.","Error");
		}
	}
}