/*
	Nabaztag API
	Copyright 2007 Shannon Hicks
	http://www.iotashan.com/nabaztagexplorer/
	
	Released for use under GPL v2
	http://www.gnu.org/copyleft/gpl.html
*/
package org.iotashan.nabaztagAPI {
	import mx.collections.ArrayCollection;
	
	/**
	 * The NabaztagChoreography class is a container used by the NabaztagAPIService
	 * to represent a complete choreography action.
	*/
	public class NabaztagChoreography {
		private var _actions:ArrayCollection;
		private var _tempo:Number;
		
		/**
		 * Constructor method.
		*/
		public function NabaztagChoreography(tempo:Number=10,actions:ArrayCollection=null) {
			_tempo = tempo;
			_actions = actions;
			// make sure there's at least an empty array
			if (!actions) {
				actions = new ArrayCollection();
			}
		}
		
		// getter/setter methods
		/**
		 * A collection of NabaztagChoreographyCommand objects
		*/
		public function get actions():ArrayCollection {
			return _actions;
		}
		
		/**
		 * @private
		*/
		public function set actions(value:ArrayCollection):void {
			_actions = value;
		}
		
		/**
		 * The tempo is expressed in Hz. A value of 10 would represent a tempo of a tenth of a second.
		*/
		public function get tempo():Number {
			return _tempo;
		}
		
		/**
		 * @private
		*/
		public function set tempo(value:Number):void {
			_tempo = value;
		}
		
		/**
		 * Gets the command at the specified index.
		*/
		public function getActionAt(index:Number):NabaztagChoreographyCommand {
			return _actions.getItemAt(index) as NabaztagChoreographyCommand;
		}
		
		/**
		 * Adds the command at the end of the collection.
		*/
		public function addAction(action:NabaztagChoreographyCommand):void {
			_actions.addItem(action);
		}
		
		/**
		 * Adds the command at the specified index.
		*/
		public function addActionAt(action:NabaztagChoreographyCommand,index:Number):void {
			_actions.addItemAt(action,index);
		}
		
		/**
		 * Removes the command at the specified index.
		*/
		public function removeActionAt(index:Number):void {
			_actions.removeItemAt(index);
		}
		
		/**
		 * Returns this choreography in the proper URL format
		 * for use with the Nabaztag API.
		*/
		public function getChoreography():String {
			var thisCommand:String = "";
			// set the tempo
			thisCommand += _tempo.toString();
			// add the actions
			for (var i:Number=0;i<_actions.length;i++) {
				thisCommand += "," + getActionAt(i).getCommand();
			}
			
			return thisCommand;
		}
	}
}