/*
	Nabaztag API
	Copyright 2007 Shannon Hicks
	http://www.iotashan.com/nabaztagexplorer/
	
	Released for use under GPL v2
	http://www.gnu.org/copyleft/gpl.html
*/
package org.iotashan.nabaztagAPI {
	/**
	 * A choreography command for moving the ears.
	*/
	public class NabaztagEarsCommand extends NabaztagChoreographyCommand {
		private var _targetEar:Number;
		private var _angle:Number;
		private var _rotation:Number;
		
		/**
		 * The string value to be used for the command
		*/
		public const EARS_COMMAND:String = "motor";
		
		// ear constants
		public const RIGHT_EAR:Number = 0;
		public const LEFT_EAR:Number = 1;
		
		// rotation constants
		public const ROTATE_FORWARD:Number = 0;
		public const ROTATE_BACKWARD:Number = 1;
		
		/**
		 * Constructor Method.
		*/
		public function NabaztagEarsCommand(startTime:Number,targetEar:Number=0,angle:Number=0,rotation:Number=0) {
			super(EARS_COMMAND,startTime);
			
			_targetEar = targetEar;
			_angle = angle;
			_rotation = rotation;
		}
		
		// getter/setter methods
		/**
		 * Target ear for this command.
		*/
		public function get targetEar():Number {
			return _targetEar;
		}
		
		/**
		 * @private
		*/
		public function set targetEar(value:Number):void {
			_targetEar = value;
		}
		
		/**
		 * Angle to move the ear to. An angle of 0 is straight up, an angle of 180 would be "flat" down.
		*/
		public function get angle():Number {
			return _angle;
		}
		
		/**
		 * @private
		*/
		public function set angle(value:Number):void {
			_angle = value;
		}
		
		/**
		 * Direction to move the ear in to get to the angle.
		*/
		public function get rotation():Number {
			return _rotation;
		}
		
		/**
		 * @private
		*/
		public function set rotation(value:Number):void {
			_rotation = value;
		}
		
		/**
		 * Returns this command in the proper URL format
		 * for use with the Nabaztag API.
		*/
		override public function getCommand():String {
			var thisCommand:String = "";
			
			// First value : action time (' l'heure ')
			thisCommand += startTime.toString() + ",";
			// Second value : 'motor', to move an ear
			thisCommand += command + ",";
			// Third value : Ears command
			thisCommand += _targetEar.toString() + ",";
			// Fourth value : Angle of ear
			thisCommand += _angle.toString() + ",";
			// Fifth value : Unused, set at "0" 
			thisCommand += "0,";
			// Sixth value : Rotation of the ears directions
			thisCommand += _rotation.toString();
			
			return thisCommand;
		}
	}
}