/*
	Nabaztag API
	Copyright 2007 Shannon Hicks
	http://www.iotashan.com/nabaztagexplorer/
	
	Released for use under GPL v2
	http://www.gnu.org/copyleft/gpl.html
*/
package org.iotashan.nabaztagAPI {
	/**
	 * A choreography command for controlling the LED lights.
	*/
	public class NabaztagLEDCommand extends NabaztagChoreographyCommand {
		private var _targetLED:Number;
		private var _selectedColor:uint;
		
		/**
		 * The string value to be used for the command
		*/
		public const LED_COMMAND:String = "led";
		
		// led constants
		public const BOTTOM_LED:Number = 0;
		public const LEFT_LED:Number = 1;
		public const MIDDLE_LED:Number = 2;
		public const RIGHT_LED:Number = 3;
		public const HIGH_LED:Number = 4;
		
		/**
		 * Constructor method.
		*/
		public function NabaztagLEDCommand(startTime:Number,targetLED:Number=0,selectedColor:uint=0x000000) {
			super(LED_COMMAND,startTime);
			
			_targetLED = targetLED;
			_selectedColor = selectedColor;
		}
		
		// getter/setter methods

		/**
		 * The target LED for this command.
		*/
		public function get targetLED():Number {
			return _targetLED;
		}
		
		/**
		 * @private
		*/
		public function set targetLED(value:Number):void {
			_targetLED = value;
		}
		
		/**
		 * Color to make the LED. You will want to use bright colors 
		 * so they show up. To turn off a LED, use the color black.
		*/
		public function get selectedColor():uint {
			return _selectedColor;
		}
		
		/**
		 * @private
		*/
		public function set selectedColor(value:uint):void {
			_selectedColor = value;
		}
		
		/**
		 * Returns this command in the proper URL format
		 * for use with the Nabaztag API.
		*/
		override public function getCommand():String {
			var thisCommand:String = "";
			
			// First value : action time (' l'heure ')
			thisCommand += startTime.toString() + ",";
			// Second value : 'motor', to move an ear
			thisCommand += command + ",";
			// Third value : To define which LED you want to illuminate.
			thisCommand += _targetLED.toString() + ",";
			// Fourth, Fifth, Sixth value : the color in RGB.
			var _red:Number = _selectedColor >> 16;
			var _green:Number = _selectedColor >> 8 & 0xFF;
			var _blue:Number = _selectedColor & 0xFF;
			thisCommand += _red.toString() + ",";
			thisCommand += _green.toString() + ",";
			thisCommand += _blue.toString();
			
			return thisCommand;
		}
	}
}